@@ -453,6 +453,8 @@ static int
 wireless_applet_timeout_handler (WirelessApplet *applet)
 {
 	if (applet->file == NULL) {
+		wireless_applet_update_state (applet,
+				applet->device, -1, -1, -1);
 		return FALSE;
 	}
 
@@ -475,6 +477,7 @@ show_error_dialog (gchar *mesg,...)
 			0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 			mesg, NULL);
 	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 	g_free (tmp);
 	va_end (ap);
 }
@@ -492,6 +495,7 @@ show_warning_dialog (gchar *mesg,...)
 			0, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
 			mesg, NULL);
 	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 	g_free (tmp);
 	va_end (ap);
 }
@@ -510,6 +514,7 @@ show_message_dialog (char *mesg,...)
 			0, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
 			mesg, NULL);
 	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 	g_free (tmp);
 	va_end (ap);
 }
@@ -551,7 +556,8 @@ wireless_applet_load_properties (WirelessApplet *applet)
 static void
 wireless_applet_save_properties (WirelessApplet *applet)
 {
-	panel_applet_gconf_set_string (PANEL_APPLET (applet),
+	if (applet->device)
+		panel_applet_gconf_set_string (PANEL_APPLET (applet),
 			"device", applet->device, NULL);
 	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
 			"percent", applet->show_percent, NULL);
@@ -563,7 +569,7 @@ static void
 wireless_applet_option_change (GtkWidget *widget, gpointer user_data)
 {
 	GtkWidget *entry;
-	GtkWidget *menu;
+	GtkWidget *menu = NULL;
 	char *str;
 	WirelessApplet *applet = (WirelessApplet *)user_data;
 
@@ -581,10 +587,13 @@ wireless_applet_option_change (GtkWidget *widget, gpointer user_data)
 			(GTK_TOGGLE_BUTTON (entry)));
 
 	entry = g_object_get_data (G_OBJECT (applet->prefs), "device-menu");
+
 	menu = gtk_menu_get_active
 		(GTK_MENU (gtk_option_menu_get_menu (GTK_OPTION_MENU (entry))));
-	str = g_object_get_data (G_OBJECT (menu), "device-selected");
-	wireless_applet_set_device (applet, str);
+	if (menu) {
+		str = g_object_get_data (G_OBJECT (menu), "device-selected");
+		wireless_applet_set_device (applet, str);
+	}
 
 	/* Save the properties */
 	wireless_applet_save_properties (applet);
@@ -738,7 +747,7 @@ static void
 wireless_applet_destroy (WirelessApplet *applet, gpointer horse)
 {
 	g_free (applet->device);
-	
+
 	g_list_foreach (applet->no_link_images, (GFunc)g_free, NULL);
 	g_list_free (applet->no_link_images);
 
@@ -751,7 +760,23 @@ wireless_applet_destroy (WirelessApplet *applet, gpointer horse)
 	g_list_foreach (applet->devices, (GFunc)g_free, NULL);
 	g_list_free (applet->devices);
 
-	fclose (applet->file);
+	if (applet->animate_timer > 0) {
+		gtk_timeout_remove (applet->animate_timer);
+		applet->animate_timer = 0;
+	}
+
+	if (applet->timeout_handler_id > 0) {
+		gtk_timeout_remove (applet->timeout_handler_id);
+		applet->timeout_handler_id = 0;
+	}
+
+	if (applet->prefs) {
+		gtk_widget_destroy (applet->prefs);
+		applet->prefs = NULL;
+	}
+
+	if (applet->file)
+		fclose (applet->file);
 }
 
 static GtkWidget *
